from unittest import TestCase
from storekit.appstore import AppStoreValidator
from storekit.errors import AppValidationError
from storekit.tests.compatibility import mock

import json
import os.path


JSON_FILE_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'response.json')


class AppStoreTest(TestCase):

    def setUp(self):
        with open(JSON_FILE_PATH, 'r') as f:
            self.response = json.loads(f.read())
        self.bundle_id = 'com.example.test'

    def tearDown(self):
        self.response = None
        self.bundle_id = None

    def test_use_sandbox(self):
        validator = AppStoreValidator(self.bundle_id, True)
        self.assertEqual(validator.url, 'https://sandbox.itunes.apple.com/verifyReceipt')

    def test_not_use_sandbox(self):
        validator = AppStoreValidator(self.bundle_id, False)
        self.assertEqual(validator.url, 'https://buy.itunes.apple.com/verifyReceipt')

    def test_use_password(self):
        with mock.patch('requests.post') as mock_post:
            mock_json = mock.Mock()
            mock_json.json.return_value = self.response
            mock_post.return_value = mock_json

            self.assertTrue(AppStoreValidator(self.bundle_id).validate('', password='test_password'))

    def test_validate_success(self):
        with mock.patch('requests.post') as mock_post:
            mock_json = mock.Mock()
            mock_json.json.return_value = self.response
            mock_post.return_value = mock_json

            self.assertTrue(AppStoreValidator(self.bundle_id).validate(''))

    def test_validate_status_error(self):
        self.response['response']['status'] = 1
        with mock.patch('requests.post') as mock_post:
            mock_json = mock.Mock()
            mock_json.json.return_value = self.response
            mock_post.return_value = mock_json

            with self.assertRaises(AppValidationError):
                _ = AppStoreValidator(self.bundle_id).validate('')

    def test_mismatch_bundle_id(self):
        self.bundle_id = 'com.example.wrong'
        with mock.patch('requests.post') as mock_post:
            mock_json = mock.Mock()
            mock_json.json.return_value = self.response
            mock_post.return_value = mock_json

            with self.assertRaises(AppValidationError):
                _ = AppStoreValidator(self.bundle_id).validate('')



from unittest import TestCase
from storekit.errors import AppValidationError


class ErrorsTest(TestCase):

    def setUp(self):
        self.error = AppValidationError(msg='')

    def tearDown(self):
        self.error = None

    def test_app_validation_error(self):
        self.error.response = {'status': 1}
        self.assertNotEqual(self.error.response, None)



from django.db import models


class InApp(models.Model):
    quantity = models.IntegerField(blank=True)
    product_id = models.CharField(blank=False, default='', max_length=255)
    transaction_id = models.IntegerField(blank=True)
    original_transaction_id = models.IntegerField(blank=True)
    purchase_date = models.CharField(blank=False, default='', max_length=255)
    purchase_date_ms = models.IntegerField(blank=True)
    purchase_date_pst = models.CharField(blank=False, default='', max_length=255)
    original_purchase_date = models.CharField(blank=False, default='', max_length=255)
    original_purchase_date_ms = models.IntegerField(blank=True)
    original_purchase_date_pst = models.CharField(blank=False, default='', max_length=255)
    is_trial_period = models.BooleanField(blank=False, default=False)

    def __init__(self, quantity, product_id, transaction_id, original_transaction_id,
                 purchase_date, purchase_date_ms, purchase_date_pst, original_purchase_date,
                 original_purchase_date_ms, original_purchase_date_pst, is_trial_period):
        models.Model.__init__(self)
        self.quantity = quantity
        self.product_id = product_id
        self.transaction_id = transaction_id
        self.original_transaction_id = original_transaction_id
        self.purchase_date = purchase_date
        self.purchase_date_ms = purchase_date_ms
        self.purchase_date_pst = purchase_date_pst
        self.original_purchase_date = original_purchase_date
        self.original_purchase_date_ms = original_purchase_date_ms
        self.original_purchase_date_pst = original_purchase_date_pst
        self.is_trial_period = is_trial_period

    def __str__(self):
        return '{}: {}'.format(self.product_id, self.quantity)

    def __repr__(self):
        return '{} <ID: {}>: "{}"'.format(self.product_id, self.id, self.__class__.__name__)

    @classmethod
    def parser(cls, json, is_save=True):
        in_app = {
            'quantity': int(json['quantity']),
            'product_id': json['product_id'],
            'transaction_id': int(json['transaction_id']),
            'original_transaction_id': int(json['original_transaction_id']),
            'purchase_date': json['purchase_date'],
            'purchase_date_ms': int(json['purchase_date_ms']),
            'purchase_date_pst': json['purchase_date_pst'],
            'original_purchase_date': json['original_purchase_date'],
            'original_purchase_date_ms': int(json['original_purchase_date_ms']),
            'original_purchase_date_pst': json['original_purchase_date_pst'],
            'is_trial_period': True if 'true' == json['is_trial_period'] else False
        }
        obj = cls(**in_app)
        if is_save:
            obj.save()
        return obj


class Receipt(models.Model):
    receipt_type = models.CharField(blank=False, default='', max_length=255)
    adam_id = models.IntegerField(blank=True)
    app_item_id = models.IntegerField(blank=True)
    bundle_id = models.CharField(blank=False, default='', max_length=255)
    application_version = models.CharField(blank=False, default='', max_length=255)
    download_id = models.IntegerField(blank=True)
    version_external_identifier = models.IntegerField(blank=True)
    receipt_creation_date = models.CharField(blank=False, default='', max_length=255)
    receipt_creation_date_ms = models.IntegerField(blank=True)
    receipt_creation_date_pst = models.CharField(blank=False, default='', max_length=255)
    request_date = models.CharField(blank=False, default='', max_length=255)
    request_date_ms = models.IntegerField(blank=True)
    request_date_pst = models.CharField(blank=False, default='', max_length=255)
    original_purchase_date = models.CharField(blank=False, default='', max_length=255)
    original_purchase_date_ms = models.IntegerField(blank=True)
    original_purchase_date_pst = models.CharField(blank=False, default='', max_length=255)
    original_application_version = models.CharField(blank=False, default='', max_length=255)
    in_app = models.ForeignKey(InApp, blank=True)

    def __init__(self, receipt_type, adam_id, app_item_id, bundle_id,
                 application_version, download_id, version_external_identifier,
                 receipt_creation_date, receipt_creation_date_ms, receipt_creation_date_pst,
                 request_date, request_date_ms, request_date_pst, original_purchase_date,
                 original_purchase_date_ms, original_purchase_date_pst, original_application_version, in_app):
        models.Model.__init__(self)
        self.receipt_type = receipt_type
        self.adam_id = adam_id
        self.app_item_id = app_item_id
        self.bundle_id = bundle_id
        self.application_version = application_version
        self.download_id = download_id
        self.version_external_identifier = version_external_identifier
        self.receipt_creation_date = receipt_creation_date
        self.receipt_creation_date_ms = receipt_creation_date_ms
        self.receipt_creation_date_pst = receipt_creation_date_pst
        self.request_date = request_date
        self.request_date_ms = request_date_ms
        self.request_date_pst = request_date_pst
        self.original_purchase_date = original_purchase_date
        self.original_purchase_date_ms = original_purchase_date_ms
        self.original_purchase_date_pst = original_purchase_date_pst
        self.original_application_version = original_application_version
        self.in_app = in_app

    def __str__(self):
        return '[{}] {}: {}'.format(self.receipt_type, self.bundle_id, self.application_version)

    def __repr__(self):
        return '<ID: {}>: "{}"'.format(self.id, self.__class__.__name__)

    @classmethod
    def parser(cls, json, in_app, is_save=True):
        receipt = {
            'receipt_type': json['receipt_type'],
            'adam_id': int(json['adam_id']),
            'app_item_id': int(json['app_item_id']),
            'bundle_id': json['bundle_id'],
            'application_version': json['application_version'],
            'download_id': int(json['download_id']),
            'version_external_identifier': int(json['version_external_identifier']),
            'receipt_creation_date': json['receipt_creation_date'],
            'receipt_creation_date_ms': int(json['receipt_creation_date_ms']),
            'receipt_creation_date_pst': json['receipt_creation_date_pst'],
            'request_date': json['request_date'],
            'request_date_ms': int(json['request_date_ms']),
            'request_date_pst': json['request_date_pst'],
            'original_purchase_date': json['original_purchase_date'],
            'original_purchase_date_ms': int(json['original_purchase_date_ms']),
            'original_purchase_date_pst': json['original_purchase_date_pst'],
            'original_application_version': json['original_application_version'],
            'in_app': InApp.parser(in_app)
        }
        obj = cls(**receipt)
        if is_save:
            obj.save()
        return obj


class Response(models.Model):
    status = models.IntegerField(blank=True)
    environment = models.CharField(blank=False, default='', max_length=255)
    receipt = models.ForeignKey(Receipt, blank=True)

    def __init__(self, status, environment, receipt):
        models.Model.__init__(self)
        self.status = status
        self.environment = environment
        self.receipt = receipt

    def __str__(self):
        return 'Response: ' + self.environment

    def __repr__(self):
        return '<ID: {}>: "{}"'.format(self.id, self.__class__.__name__)

    @classmethod
    def parser(cls, json, in_app, is_save=True):
        response = {
            'status': json['status'],
            'environment': json['environment'],
            'receipt': Receipt.parser(json['receipt'], in_app)
        }
        obj = cls(**response)
        if is_save:
            obj.save()
        return obj


class Purchase(models.Model):
    transaction_id = models.IntegerField(blank=True)
    product_id = models.CharField(blank=False, default='', max_length=255)
    quantity = models.IntegerField(blank=False, default=0)
    purchased_at = models.CharField(blank=False, default='', max_length=255)
    response = models.ForeignKey(Response, blank=True)

    def __init__(self, transaction_id, product_id, quantity, purchased_at, response):
        models.Model.__init__(self)
        self.transaction_id = transaction_id
        self.product_id = product_id
        self.quantity = quantity
        self.purchased_at = purchased_at
        self.response = response

    def __str__(self):
        return 'Purchase: ' + str(self.transaction_id)

    def __repr__(self):
        return '{} <ID: {}>: "{}"'.format(self.product_id, self.id, self.__class__.__name__)

    @classmethod
    def parser(cls, receipt, response, in_app, is_save=True):
        purchase = {
            'transaction_id': receipt['transaction_id'],
            'product_id': receipt['product_id'],
            'quantity': int(receipt['quantity']),
            'purchased_at': receipt['purchased_at'],
            'response': Response.parser(response, in_app)
        }
        obj = cls(**purchase)
        if is_save:
            obj.save()
        return obj



from django.conf import settings
from .appstore import AppStoreValidator
from .errors import AppValidationError
from .models import Purchase
import logging


def normal_receipt(receipt, sandbox):
    try:
        validator = AppStoreValidator(
            bundle_id=settings.STOREKIT_APP_BUNDLE_ID,
            sandbox=sandbox
        )

        try:
            return validator.validate(
                receipt=receipt
            )

        except AppValidationError as e:
            logging.error(e)
            raise

    except AttributeError as e:
        # NotFound STOREKIT_APP_BUNDLE_ID
        logging.error(e)
        raise


def subscribe_receipt(receipt, sandbox):
    try:
        validator = AppStoreValidator(
            bundle_id=settings.STOREKIT_APP_BUNDLE_ID,
            sandbox=sandbox
        )

        try:
            return validator.validate(
                receipt=receipt,
                password=settings.STOREKIT_PURCHASED_SECRET
            )

        except AppValidationError as e:
            logging.error(e)
            raise

        except AttributeError as e:
            # NotFound STOREKIT_PURCHASED_SECRET
            logging.error(e)
            raise

    except AttributeError as e:
        # NotFound STOREKIT_APP_BUNDLE_ID
        logging.error(e)
        raise



from django.conf import settings
from unittest import TestCase
from storekit.errors import AppValidationError
from storekit.receipt import normal_receipt, subscribe_receipt
from storekit.tests.compatibility import mock

import json
import os.path

JSON_FILE_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'response.json')


class NormalReceiptTest(TestCase):

    def setUp(self):
        with open(JSON_FILE_PATH, 'r') as f:
            self.response = json.loads(f.read())
        self.bundle_id = settings.STOREKIT_APP_BUNDLE_ID
        self.sandbox = False

    def tearDown(self):
        self.response = None
        self.bundle_id = None
        self.sandbox = None

    def test_success(self):
        with mock.patch('requests.post') as mock_post:
            mock_json = mock.Mock()
            mock_json.json.return_value = self.response
            mock_post.return_value = mock_json

            self.assertTrue(normal_receipt(receipt='', sandbox=self.sandbox))

    def test_failure_status_code(self):
        self.response['response']['status'] = 1
        with mock.patch('requests.post') as mock_post:
            mock_json = mock.Mock()
            mock_json.json.return_value = self.response
            mock_post.return_value = mock_json

            with self.assertRaises(AppValidationError):
                normal_receipt(receipt='', sandbox=self.sandbox)


class SubscribeReceiptTest(TestCase):

    def setUp(self):
        with open(JSON_FILE_PATH, 'r') as f:
            self.response = json.loads(f.read())
        self.bundle_id = settings.STOREKIT_APP_BUNDLE_ID
        self.sandbox = False

    def tearDown(self):
        self.response = None
        self.bundle_id = None
        self.sandbox = None

    def test_success(self):
        with mock.patch('requests.post') as mock_post:
            mock_json = mock.Mock()
            mock_json.json.return_value = self.response
            mock_post.return_value = mock_json

            self.assertTrue(subscribe_receipt(receipt='', sandbox=self.sandbox))

    def test_failure_status_code(self):
        self.response['response']['status'] = 1
        with mock.patch('requests.post') as mock_post:
            mock_json = mock.Mock()
            mock_json.json.return_value = self.response
            mock_post.return_value = mock_json

            with self.assertRaises(AppValidationError):
                subscribe_receipt(receipt='', sandbox=self.sandbox)



from requests.exceptions import RequestException
from .errors import AppValidationError
from .models import Purchase
import requests


api_result_ok = 0
api_result_errors = {
    21000: 'Bad json',
    21002: 'Bad data',
    21003: 'Receipt authentication',
    21004: 'Shared secret mismatch',
    21005: 'Server is unavailable',
    21006: 'Subscription has expired',
    21007: 'Sandbox receipt was sent to the production env',
    21008: 'Production receipt was sent to the sandbox env',
}


class AppStoreValidator(object):

    def __init__(self, bundle_id, sandbox=False):
        self.bundle_id = bundle_id

        if sandbox:
            self.url = 'https://sandbox.itunes.apple.com/verifyReceipt'
        else:
            self.url = 'https://buy.itunes.apple.com/verifyReceipt'

    def validate(self, receipt, password=None):
        receipt_json = {'receipt-data': receipt}
        if password:
            receipt_json['password'] = password

        api_response = requests.post(self.url, json=receipt_json).json()

        status = api_response['response']['status']

        if status != api_result_ok:
            error = AppValidationError(api_result_errors.get(status, 'Unknown API status'), api_response)
            raise error

        response = api_response['response']
        purchases = self._parse_receipt(api_response, response)
        return purchases

    def _parse_receipt(self, api_response, response):
        if self.bundle_id != response['receipt']['bundle_id']:
            error = AppValidationError('Bundle id mismatch', api_response)
            raise error
        return [Purchase.parser(api_response, response, in_app) for in_app in response['receipt']['in_app']]



from unittest import TestCase
from storekit.models import InApp, Receipt, Response, Purchase

import json
import os.path


JSON_FILE_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'response.json')


class InAppTest(TestCase):

    def setUp(self):
        with open(JSON_FILE_PATH, 'r') as f:
            self.response = json.loads(f.read())['response']['receipt']['in_app'][0]

    def tearDown(self):
        self.response = None

    def test_new_data_save_true(self):
        in_app = InApp.parser(
            self.response
        )
        self.assertTrue(in_app)

    def test_new_data_save_false(self):
        in_app = InApp.parser(
            self.response,
            is_save=False
        )
        self.assertTrue(in_app)

    def test_str(self):
        in_app = InApp.parser(
            self.response
        )
        self.assertEqual(str(in_app), '{}: {}'.format(in_app.product_id, in_app.quantity))

    def test_repr(self):
        in_app = InApp.parser(
            self.response
        )
        self.assertEqual(repr(in_app),
                         '{} <ID: {}>: "{}"'.format(in_app.product_id, in_app.id, in_app.__class__.__name__))


class ReceiptTest(TestCase):

    def setUp(self):
        with open(JSON_FILE_PATH, 'r') as f:
            self.response = json.loads(f.read())['response']['receipt']
            self.in_app = self.response['in_app'][0]

    def tearDown(self):
        self.response = None
        self.in_app = None

    def test_new_data_save_true(self):
        receipt = Receipt.parser(
            self.response,
            in_app=self.in_app
        )
        self.assertTrue(receipt)

    def test_new_data_save_false(self):
        receipt = Receipt.parser(
            self.response,
            in_app=self.in_app,
            is_save=False
        )
        self.assertTrue(receipt)

    def test_str(self):
        receipt = Receipt.parser(
            self.response,
            in_app=self.in_app
        )
        self.assertEqual(str(receipt),
                         '[{}] {}: {}'.format(receipt.receipt_type, receipt.bundle_id, receipt.application_version))

    def test_repr(self):
        receipt = Receipt.parser(
            self.response,
            in_app=self.in_app
        )
        self.assertEqual(repr(receipt),
                         '<ID: {}>: "{}"'.format(receipt.id, receipt.__class__.__name__))


class ResponseTest(TestCase):

    def setUp(self):
        with open(JSON_FILE_PATH, 'r') as f:
            self.response = json.loads(f.read())['response']
            self.in_app = self.response['receipt']['in_app'][0]

    def tearDown(self):
        self.response = None
        self.in_app = None

    def test_new_data_save_true(self):
        response = Response.parser(
            self.response,
            in_app=self.in_app
        )
        self.assertTrue(response)

    def test_new_data_save_false(self):
        response = Response.parser(
            self.response,
            in_app=self.in_app,
            is_save=False
        )
        self.assertTrue(response)

    def test_str(self):
        response = Response.parser(
            self.response,
            in_app=self.in_app
        )
        self.assertEqual(str(response),
                         'Response: ' + response.environment)

    def test_repr(self):
        response = Response.parser(
            self.response,
            in_app=self.in_app
        )
        self.assertEqual(repr(response),
                         '<ID: {}>: "{}"'.format(response.id, response.__class__.__name__))


class PurchaseTest(TestCase):

    def setUp(self):
        with open(JSON_FILE_PATH, 'r') as f:
            self.receipt = json.loads(f.read())
            self.response = self.receipt['response']
            self.in_app = self.response['receipt']['in_app'][0]

    def tearDown(self):
        self.response = None
        self.in_app = None

    def test_new_data_save_true(self):
        purchase = Purchase.parser(
            self.receipt,
            self.response,
            in_app=self.in_app
        )
        self.assertTrue(purchase)

    def test_new_data_save_false(self):
        purchase = Purchase.parser(
            self.receipt,
            self.response,
            in_app=self.in_app,
            is_save=False
        )
        self.assertTrue(purchase)

    def test_str(self):
        purchase = Purchase.parser(
            self.receipt,
            self.response,
            in_app=self.in_app
        )
        self.assertEqual(str(purchase),
                         'Purchase: ' + str(purchase.transaction_id))

    def test_repr(self):
        purchase = Purchase.parser(
            self.receipt,
            self.response,
            in_app=self.in_app
        )
        self.assertEqual(repr(purchase),
                         '{} <ID: {}>: "{}"'.format(purchase.product_id, purchase.id, purchase.__class__.__name__))
